var LibPrepDilutionProtocol = function()
{
	var lpdp = {};
	var debug = 0;
	
	lpdp.TITLE = 'MeLuDI';
	var MINIMAL_DNA_VOLUME = 1.0; //µl
	var COLORS = ['blue', 'green', 'yellow', 'red'];
	var nextColor = 0;
	var meludiVersion;

	// Page initialization
	lpdp.initPage = function()
	{
		Buttons.addClickHandler('print-button', Wizard.goPrint);
		var numItems = Data.get('page-data', 'numitems');
		var submitInfoJsonStr = Data.get('page-data', 'submitinfo');
		meludiVersion = Data.get('page-data', 'meludiversion');
		lpdp.initializeProtocol(submitInfoJsonStr);
	}

	lpdp.initializeProtocol = function(submitInfoJsonStr)
	{
		var frm = document.forms['meludi'];

		var submitInfo = JSON.parse(submitInfoJsonStr);
		var docMode = submitInfo.docMode;
		var itemInfo = submitInfo.items;
		var startPlateName = submitInfo.startPlateName;
		var dilutionProtocolDate = submitInfo.dilutionProtocolDate;

		//var protocolVersionNo = submitInfo.protocolVersionNo;
		var protocolVersionNo = '1.0';
		var protocolPreviewFlag = submitInfo.protocolPreviewFlag;
		var numQiacubeItems = itemInfo.length;

		// Header data
		var docModeText = 'Protocol';
		if (docMode == null || docMode == '')
		{
			docMode = 'protocol';
		}
		if (docMode && docMode == 'report')
		{
			docModeText = 'Report';
		}
		if (protocolVersionNo == null || protocolVersionNo == '')
		{
			protocolVersionNo = '?';
		}
		var protocolPreviewText = '';
		if (protocolPreviewFlag != null && protocolPreviewFlag == 'true')
		{
			protocolPreviewText = ' <i>(preview)</i>';
		}
		var headerText = 'Dilution ' + docModeText + ' for library preparation v' + protocolVersionNo + protocolPreviewText + ' <span class="meludi">' + LibPrepDilutionProtocol.TITLE + ' ' + meludiVersion + '</span>';
		var protocolDateNameText = 'Dilution ' + docMode + ' date';
		Doc.element('libraryPreparationDilutionProtocolHeader').innerHTML = headerText;
		Doc.element('protocolDateName').innerHTML = protocolDateNameText;

		Doc.element('startplateid').innerHTML = Strings.encodeTags(startPlateName);
		Doc.element('dilutionProtocolDate').innerHTML = Strings.encodeTags(dilutionProtocolDate);

		// Item data
		var accumItemNo = 0;
		accumItemNo = tableRowsForItems(accumItemNo, itemInfo, 'Item', 'item');

		Doc.show('all-protocol');
	}
	
	function tableRowsForItems(
		accumItemNo,
		itemJsonArr,
		itemType,
		itemTypeVarName)
	{
		// Table item row
		var html = '';
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
				var dnaStorageBox = '-';
				var fpaPos = '-';
				var fpbPos = '-';
				var dnaConc = null;
				var dnaDeltaCt = null;
				var dilutionFactor = null;
				var volAliquotSolution = null;
				var volDnaInSolution = null;
				var waterInSolution = null;
				var solutionVolume = null;
				var dnaComment = null;
				var comment = null;
				if (item)
				{
					// DNA extract from specimen
					var dna = item.info;
					if (dna)
					{
						if (dna.origBioWell)
						{
							dnaStorageBox = Strings.encodeTags(dna.origBioWell.bioPlate.name + '[' + dna.origBioWell.location + ']');
						}
						fpaPos = item.fpaPos;
						fpbPos = item.fpbPos;
						dnaConc = dna.origQubitConc;
						dnaDeltaCt = dna.origDeltaCt;
						dnaComment = dna.origDescription;
						dilutionFactor = dna.dilutionFactor;
						//volAliquotSolution = dna.volAliquotSolution;
						volDnaInSolution = 2 * dna.volDnaInSolution;
						waterInSolution = 2 * dna.waterInSolution;
						solutionVolume = 2 * dna.volAliquotSolution;
					}

					// Input DNA item storage location should be displayed in the DNA extract storage location column
					if (lpdp.isDna(item) && item.origBioWell != null)
					{
						dnaStorageBox = Strings.encodeTags(item.origBioWell.bioPlate.name + '[' + item.origBioWell.location + ']');
					}
					comment = item.comment;
				}
				if (item.operatorDeliveryComment)
				{
					comment = item.operatorDeliveryComment;
				}
				if (dnaComment)
				{
					if (comment)
					{
						comment += ' ';
					}
					else
					{
						comment = '';
					}
					comment += 'DNA: ' + dnaComment;
				}
				var waterVolumeStr = '-';
				if (waterInSolution > 0)
				{
					waterVolumeStr = lpdp.encodeFloatTags(Numbers.formatNumber(waterInSolution, 2));
				}
				// Insert item data in table columns
				//Doc.element('extract.'+accumItemNo).innerHTML = Strings.encodeTags(item.info.name);
				Doc.element('extract.'+accumItemNo).innerHTML = Strings.encodeTags(item.info.origName);
				Doc.element('dnaStorageBox.'+accumItemNo).innerHTML = Strings.encodeTags(dnaStorageBox);
				Doc.element('fpaPos.'+accumItemNo).innerHTML = Strings.encodeTags(fpaPos);
				Doc.element('fpbPos.'+accumItemNo).innerHTML = Strings.encodeTags(fpbPos);
				if (dna.volDnaInSolution < MINIMAL_DNA_VOLUME)
				{
					Doc.element('dnaVolume.'+accumItemNo).innerHTML = '<span class="lowvolume">' + lpdp.encodeFloatTags(Numbers.formatNumber(volDnaInSolution, 2)) + '</span>';
					Doc.element('waterVolume.'+accumItemNo).innerHTML = '<span class="lowvolume">' + Strings.encodeTags(waterVolumeStr) + '</span>';
					Doc.element('solutionVolume.'+accumItemNo).innerHTML = '<span class="lowvolume">' + lpdp.encodeFloatTags(Numbers.formatNumber(solutionVolume, 2)) + '</span>';
				}
				else
				{
					Doc.element('dnaVolume.'+accumItemNo).innerHTML = lpdp.encodeFloatTags(Numbers.formatNumber(volDnaInSolution, 2));
					Doc.element('waterVolume.'+accumItemNo).innerHTML = Strings.encodeTags(waterVolumeStr);
					Doc.element('solutionVolume.'+accumItemNo).innerHTML = lpdp.encodeFloatTags(Numbers.formatNumber(solutionVolume, 2));
				}
				Doc.element('dilutionFactor.'+accumItemNo).innerHTML = lpdp.encodeFloatTags(Numbers.formatNumber(dilutionFactor, 4));
				Doc.element('deltaCt.'+accumItemNo).innerHTML = lpdp.encodeFloatTags(Numbers.formatNumber(dnaDeltaCt, 2));
				Doc.element('dnaConc.'+accumItemNo).innerHTML = lpdp.encodeFloatTags(Numbers.formatNumber(dnaConc, 2));
				Doc.element('remark.'+accumItemNo).innerHTML = Strings.encodeTags(comment);
				
				// Increment accumulated item number
				accumItemNo++;
			}
		}
		return accumItemNo;
	}

	lpdp.isDna = function(item)
	{
		var isDna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".d"
				if (name.indexOf('.d') == (name.length - 2))
				{
					isDna = true;
				}
			}
		}
		return isDna;
	}

	lpdp.encodeFloatTags = function(value)
	{
		var encodedValue = value;
		if (!value)
		{
			encodedValue = Strings.encodeTags(value);
		}
		return encodedValue;
	}

	// A biowell as plate name
	lpdp.asBioPlateName = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		if (!plate) return '';
		var text = plate.name;
		return text;
	}

	// A biowell as the location
	lpdp.asPureBioPlateLocation = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		var text = well.location;
		if (plate.storage)
		{
			var storage = plate.storage;
			var tmp = [];
			if (storage.name) tmp[tmp.length] = storage.name;
			if (storage.section) tmp[tmp.length] = 'section: ' +storage.section;
			if (storage.tray) tmp[tmp.length] = 'tray: ' +storage.tray;
			if (storage.position) tmp[tmp.length] = 'position: ' +storage.position;
			text = text + ' ' + tmp.join('; ');
		}
		return text;
	}

	return lpdp;
}();

Doc.onLoad(LibPrepDilutionProtocol.initPage);

