var SpecimenTubeReg = function()
{
	var streg = {};
	var debug = 0;
	
	var caseInfo;
	
	var nofTubesIsValid = false;
	var samplingDateIsValid = true;
	var viableTumourCellsPercentIsValid = true;

	// Well validation variable for input specimen, DNA, RNA
	var wellIsValid = [];
	// Well validation variables for extracts from specimen
	var dnaWellIsValid = [];
	var rnaWellIsValid = [];
	
	var restartTimer;
	var restartAt;
	
	// Page initialization
	streg.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', streg.validateStep1);
		Events.addEventHandler('caseName', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', streg.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', streg.validateStep2);
		Events.addEventHandler('nofTubes', 'keypress', Events.integerOnly);
		Events.addEventHandler('nofTubes', 'blur', streg.nofTubesOnChange);
		Events.addEventHandler('arrivalDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('samplingDate', 'blur', streg.samplingDateOnChange);
		Events.addEventHandler('projectFocus', 'change', streg.projectFocusOnChange);
		Events.addEventHandler('site', 'change', streg.siteOnChange);
		Events.addEventHandler('specimenType', 'change', streg.specimenTypeOnChange);
		Events.addEventHandler('viableTumourCellsPercent', 'keypress', Events.integerOnly);
		Events.addEventHandler('viableTumourCellsPercent', 'blur', streg.viableTumourCellsPercentOnChange);
		Events.addEventHandler('otherPathNote', 'keypress', Wizard.goNextOnTab);
		Buttons.addClickHandler('selectAllMutationAnalysis', streg.mutationAnalysisSelectAll);

		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', streg.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', streg.validateStep3);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('gostop', streg.stopAutoRestart);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', streg.submit);

		var url = '../LabelGeneration.servlet?ID='+App.getSessionId();
		url += '&cmd=CountCases';
		Wizard.showLoadingAnimation('Loading cases...');
		Wizard.asyncJsonRequest(url, streg.caseInfoLoadedStep1);
	}

	streg.caseInfoLoadedStep1 = function(response)
	{
		var frm = document.forms['meludi'];	

		var nofCases = response.count;
		if (nofCases == null)
		{
			nofCases = 0;
		}
		var startNum = nofCases + 1;
		// Pad number with leading zeros to get 5 digits
		startNum = String('00000' + startNum).slice(-5);
		var startCase = 'ML' + startNum;
		frm.caseName.value = startCase;
		
		Doc.show('step-1');
		Doc.show('gonext');
	}

	streg.validateStep1 = function(event)
	{
		var frm = document.forms['meludi'];
		
		var caseName = frm.caseName.value;
		var caseIsValid = false;
		Wizard.setInputStatus('caseName');
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
		}
		else if (!Meludi.isValidCaseName(caseName))
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Only name = "ML" + 5 digits is allowed.');
			frm.caseName.focus();
		}
		else
		{
			Wizard.setInputStatus('caseName', 'valid');
			caseIsValid = true;
		}
		
		// If not valid, we prevent the wizard from moving to the next step
		if (!caseIsValid) event.preventDefault();
	}
	
	streg.initializeStep2 = function()
	{
		var frm = document.forms['meludi'];	
		var caseName = frm.caseName.value;
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo';
		url += '&caseName=' + encodeURIComponent(caseName);
		
		Wizard.showLoadingAnimation('Loading information about case...');
		Wizard.asyncJsonRequest(url, streg.caseInfoLoaded);
	}

	streg.caseInfoLoaded = function(response)
	{
		var frm = document.forms['meludi'];	
		caseInfo = response.caseInfo;
		
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('gonext');
		
		frm.arrivalDate.value = Dates.formatDate(new Date(), 'yyyyMMdd');

		frm.nofTubes.focus();
		frm.nofTubes.setSelectionRange(0,1);
	}
	
	streg.nofTubesOnChange = function()
	{
		nofTubesIsValid = false;

		var frm = document.forms['meludi'];
		var nofTubes = frm.nofTubes.value;
		
		if (nofTubes == '')
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Missing');
			return;
		}
		if (parseInt(nofTubes) < 1)
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Must be at least 1');
			return;		
		}
		Wizard.setInputStatus('nofTubes', 'valid');
		nofTubesIsValid = true; 
	}

	streg.samplingDateOnChange = function()
	{
		var frm = document.forms['meludi'];
		
		samplingDateIsValid = false;
		Wizard.setInputStatus('samplingDate');
		
		var samplingDate = frm.samplingDate.value;

		if (samplingDate != '')
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			samplingDate = Meludi.autoFillDate(samplingDate);
			frm.samplingDate.value = samplingDate;
		
			var d = Dates.parseString(samplingDate, 'yyyyMMdd');
			if (d == null)
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid date');
				return;
			}
			
			var today = new Date();
			if (d.getTime() > today.getTime())
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Future date is not valid');
				return;
			}

			var arrivalDate = frm.arrivalDate.value;
			if (arrivalDate != '' && arrivalDate < samplingDate)
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Arrival date is before sampling date');
				return;
			}

			Wizard.setInputStatus('samplingDate', 'valid');
		}
		samplingDateIsValid = true;
	}
	
	streg.projectFocusOnChange = function()
	{
		var frm = document.forms['meludi'];
		var projectFocus = frm.projectFocus.value;

		streg.mutationAnalysisReset();
		
		if (projectFocus == 'melanoma')
		{
			frm.mutationAnalysisNras.checked = true;
			frm.mutationAnalysisBraf.checked = true;
			frm.mutationAnalysisKit.checked = true;
			return;
		}
		if (projectFocus == 'lungcancer')
		{
			frm.mutationAnalysisKras.checked = true;
			frm.mutationAnalysisEgfr.checked = true;
			return;
		}
		if (projectFocus == 'coloncancer')
		{
			frm.mutationAnalysisKras.checked = true;
			frm.mutationAnalysisNras.checked = true;
			frm.mutationAnalysisBraf.checked = true;
			return;
		}
		if (projectFocus == 'GIST')
		{
			frm.mutationAnalysisPdgfra.checked = true;
			frm.mutationAnalysisKit.checked = true;
			return;
		}
	}

	streg.mutationAnalysisReset = function()
	{
		var frm = document.forms['meludi'];

		frm.mutationAnalysisKras.checked = false;
		frm.mutationAnalysisNras.checked = false;
		frm.mutationAnalysisBraf.checked = false;
		frm.mutationAnalysisEgfr.checked = false;
		frm.mutationAnalysisAlkEml4.checked = false;
		frm.mutationAnalysisPdgfra.checked = false;
		frm.mutationAnalysisKit.checked = false;
		frm.mutationAnalysisOther.checked = false;
	}

	streg.mutationAnalysisSelectAll = function()
	{
		var frm = document.forms['meludi'];

		frm.mutationAnalysisKras.checked = true;
		frm.mutationAnalysisNras.checked = true;
		frm.mutationAnalysisBraf.checked = true;
		frm.mutationAnalysisEgfr.checked = true;
		frm.mutationAnalysisAlkEml4.checked = true;
		frm.mutationAnalysisPdgfra.checked = true;
		frm.mutationAnalysisKit.checked = true;
		//frm.mutationAnalysisOther.checked = true;
	}

	streg.viableTumourCellsPercentOnChange = function()
	{
		viableTumourCellsPercentIsValid = false;

		var frm = document.forms['meludi'];
		var viableTumourCellsPercent = frm.viableTumourCellsPercent.value;
		
		if (viableTumourCellsPercent != null)
		{
			if (parseInt(viableTumourCellsPercent) < 0)
			{
				Wizard.setInputStatus('viableTumourCellsPercent', 'invalid', 'Must be at least 0');
				return;		
			}
			if (parseInt(viableTumourCellsPercent) > 100)
			{
				Wizard.setInputStatus('viableTumourCellsPercent', 'invalid', 'Must be at most 100');
				return;		
			}
		}
		Wizard.setInputStatus('viableTumourCellsPercent', 'valid');
		viableTumourCellsPercentIsValid = true; 
	}

	streg.validateStep2 = function(event)
	{
		var valid = true;
		var frm = document.forms['meludi'];	

		valid &= nofTubesIsValid;
		valid &= Wizard.isValid('arrivalDate');
		valid &= samplingDateIsValid;
		valid &= viableTumourCellsPercentIsValid;
		
		if (!valid) event.preventDefault();
	}

	streg.initializeStep3 = function()
	{
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		var nofTubes = parseInt(frm.nofTubes.value);
		nofTubes = streg.fixNofTubes(tubeContentType, nofTubes);
		frm.nofTubes.value = nofTubes;
		var storageBoxSuffix = streg.fetchStorageBoxSuffix(tubeContentType);
		if (nofTubes > 2) Doc.addClass('step-2', 'auto-hide');
		
		var html = '';

		// Add buttons for adding standard comments for all specimens/RNA/DNA
		html += '<tr class="section-header">';
		html += '<td colspan="4">Standard comments</td>';
		html += '</tr>';

		html += '<tr class="align-top">';
		html += '<td class="subprompt"></td>';
		html += '<td><div class="button basicbutton interactable" id="addCarvedOutComment" style="font-weight: normal;">Add &quot;Utkarvad&quot; (Carved out)</div></td>';
		html += '<td class="status" id="addCarvedOutComment.status"></td>';
		html += '<td class="help"><span id="addCarvedOutComment.message" class="message"></span>Click button to add standard comment for all items.</td>';		
		html += '</tr>';

		html += '<tr class="align-top">';
		html += '<td class="subprompt"></td>';
		html += '<td><div class="button basicbutton interactable" id="addMnbComment" style="font-weight: normal;">Add &quot;MNB&quot; (Medium needle biopsy)</div></td>';
		html += '<td class="status" id="addMnbComment.status"></td>';
		html += '<td class="help"><span id="addMnbComment.message" class="message"></span>Click button to add standard comment for all items.</td>';		
		html += '</tr>';

		for (var i=0; i < nofTubes; i++)
		{
			var tubeContentTypeName = streg.fetchTubeContentTypeName(tubeContentType, i);
			var tubeContentTypeNameInternal = streg.fetchTubeContentTypeNameInternal(tubeContentType, tubeContentTypeName);
			var tubeName = streg.fetchTubeName(tubeContentType, caseInfo, i);
			var tubeNo = i+caseInfo.specimenFirstIndex;
			var tubeIdentifier = streg.fetchTubeIdentifier(tubeContentType, tubeName, tubeNo);
			html += '<tr class="section-header">';
			html += '<td colspan="4">'+tubeContentTypeName+' Tube '+tubeName+'</td>';
			html += '</tr>';

/*
			var boxMsg = 'Box-number where the '+tubeContentTypeNameInternal+' tube is located in.';
*/
			var boxMsg = 'Box-number and position where the '+tubeContentTypeNameInternal+' tube is located.';
			var dnaBoxMsg = 'Box-number and position where the DNA extract will be located.';
			var rnaBoxMsg = 'Box-number and position where the RNA extract will be located.';
			var disabledAttribute = '';
			if (i == 0 && tubeContentType == 'Specimen')
			{
				boxMsg = 'No storage space allocated for first '+tubeContentTypeNameInternal+'; all amount expected to be used.';
				disabledAttribute = ' disabled';
			}	
			html += '<tr>';
			html += '<td class="subprompt">'+tubeContentTypeName+'&nbsp;'+tubeName+'&nbsp;&nbsp;&nbsp;'+'Box</td>';
			html += '<td class="input">';
/*
			html += '<input type="text" name="box.'+i+'" id="box.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'></td>';
*/
			html += '<input type="text" name="box.'+i+'" id="box.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'>';
			html += '&nbsp;Row<input type="text" name="row.'+i+'" id="row.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '&nbsp;Column<input type="text" name="column.'+i+'" id="column.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '</td>';
			html += '<td class="status" id="box.'+i+'.status"></td>';
			html += '<td class="help"><span id="box.'+i+'.message" class="message"></span>'+boxMsg+'<span id="position.'+i+'.message" class="message"></span></td>';
			html += '</tr>';
/*			
			html += '<tr>';
			html += '<td class="subprompt">Row</td>';
			html += '<td class="input">';
			html += '<input type="text" name="row.'+i+'" id="row.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '&nbsp;Column<input type="text" name="column.'+i+'" id="column.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '</td>';
			html += '<td class="status" id="position.'+i+'.status"></td>';
			html += '<td class="help"><span id="position.'+i+'.message" class="message"></span></td>';		
			html += '</tr>';
*/

			if (tubeContentType == 'Specimen')
			{
				disabledAttribute = '';
				html += '<tr>';
				html += '<td class="subprompt">DNA extract'+'&nbsp;'+tubeName+'.d&nbsp;&nbsp;&nbsp;'+'Box</td>';
				html += '<td class="input">';
				html += '<input type="text" name="dnaBox.'+i+'" id="dnaBox.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'>';
				html += '&nbsp;Row<input type="text" name="dnaRow.'+i+'" id="dnaRow.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
				html += '&nbsp;Column<input type="text" name="dnaColumn.'+i+'" id="dnaColumn.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
				html += '</td>';
				html += '<td class="status" id="dnaBox.'+i+'.status"></td>';
				html += '<td class="help"><span id="dnaBox.'+i+'.message" class="message"></span>'+dnaBoxMsg+'<span id="dnaPosition.'+i+'.message" class="message"></span></td>';
				html += '</tr>';

				html += '<tr>';
				html += '<td class="subprompt">RNA extract'+'&nbsp;'+tubeName+'.r&nbsp;&nbsp;&nbsp;'+'Box</td>';
				html += '<td class="input">';
				html += '<input type="text" name="rnaBox.'+i+'" id="rnaBox.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'>';
				html += '&nbsp;Row<input type="text" name="rnaRow.'+i+'" id="rnaRow.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
				html += '&nbsp;Column<input type="text" name="rnaColumn.'+i+'" id="rnaColumn.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
				html += '</td>';
				html += '<td class="status" id="rnaBox.'+i+'.status"></td>';
				html += '<td class="help"><span id="rnaBox.'+i+'.message" class="message"></span>'+rnaBoxMsg+'<span id="rnaPosition.'+i+'.message" class="message"></span></td>';
				html += '</tr>';

				html += '<tr>';
				html += '<td class="subprompt">Number of sections</td>';
				html += '<td class="input">';
				html += '<input type="text" name="nofSections.'+i+'" id="nofSections.'+i+'" style="width: 3em;" maxlength="2" value="1">';
				html += '</td>';
				html += '<td class="status" id="nofSections.'+i+'.status"></td>';
				html += '<td class="help"><span id="nofSections.'+i+'.message" class="message"></span>Number of sections (slices) in FFPE block.</td>';		
				html += '</tr>';
			}

			html += '<tr class="align-top">';
			html += '<td class="subprompt">Operator delivery comment</td>';
			html += '<td class="input"><textarea rows="3" name="comment.'+i+'" id="comment.'+i+'"></textarea></td>';
			html += '<td class="status" id="comment.'+i+'.status"></td>';
			html += '<td class="help"><span id="comment.'+i+'.message" class="message"></span>In this field, enter information specific for '+tubeContentTypeNameInternal+' tube ' +tubeIdentifier+'.</td>';		
			html += '</tr>';
		}
		Doc.element('specimen-tubes').innerHTML = html;

		// Add click handlers for buttons for adding standard comments for all specimens/RNA/DNA
		var addCarvedOutCommentGeneralButton = Doc.element('addCarvedOutComment');
		Buttons.addClickHandler(addCarvedOutCommentGeneralButton, streg.addCommentGeneral, {'nofTubes': nofTubes, 'text-field': 'comment.', 'text': 'Utkarvad.'});
		var addMnbCommentGeneralButton = Doc.element('addMnbComment');
		Buttons.addClickHandler(addMnbCommentGeneralButton, streg.addCommentGeneral, {'nofTubes': nofTubes, 'text-field': 'comment.', 'text': 'MNB.'});

		Wizard.setCurrentStep(3);
		Doc.show('goregister');
		Doc.show('gocancel');
		frm['comment.0'].focus();
		
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=FindStoragePositions';	
		url += '&nofTubes='+frm.nofTubes.value;
		url += '&storageBoxSuffix='+storageBoxSuffix;
		
		Wizard.showLoadingAnimation('Looking for empty box positions...');
		Wizard.asyncJsonRequest(url, streg.storagePositionsLoaded);
	}

	streg.addCommentGeneral = function(event)
	{
		var nofTubes = Data.get(event.currentTarget, 'nofTubes', event.currentTarget.id);
		var textFieldBase = Data.get(event.currentTarget, 'text-field', event.currentTarget.id);
		var text = Data.get(event.currentTarget, 'text', event.currentTarget.id);
		for (var i=0; i < nofTubes; i++)
		{
			var textField = Doc.element(textFieldBase+i);
			if (textField)
			{
				if (text && text != '')
				{
					if (textField.value != '')
					{
						textField.value += ' ';
					}
					textField.value += text;
				}
			}
		}
		streg.setNumberOfSectionsFromComment(event);
	}
	
	streg.addComment = function(event)
	{
		var textField = Doc.element(Data.get(event.currentTarget, 'text-field', event.currentTarget.id));
		var text = Data.get(event.currentTarget, 'text', event.currentTarget.id);
		if (textField)
		{
			if (text && text != '')
			{
				if (textField.value != '')
				{
					textField.value += ' ';
				}
				textField.value += text;
			}
		}
		streg.setNumberOfSectionsFromComment(event);
	}

	streg.setNumberOfSectionsFromComment = function(event)
	{
		var frm = document.forms['meludi'];
		var nofTubes = Data.get(event.currentTarget, 'nofTubes', event.currentTarget.id);
		var textFieldBase = Data.get(event.currentTarget, 'text-field', event.currentTarget.id);
		for (var i=0; i < nofTubes; i++)
		{
			var numSections = 1;
			var textField = Doc.element(textFieldBase+i);
			if (textField)
			{
				var text = textField.value;
				if (text && text != '')
				{
					// If comment "Utkarvad." and not "MNB." => number of sections set to 0
					if (text.indexOf('Utkarvad.') >= 0 && text.indexOf('MNB.') < 0)
					{
						numSections = 0;
					}
				}
			}
			frm['nofSections.'+i].value = numSections;
		}
	}
	
	streg.storagePositionsLoaded = function(response)
	{
		var freeWells = response.wells;
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		var nofTubes = parseInt(frm.nofTubes.value);

		// No storage space is allocated for first specimen,
		// since all amount is expected to be used for analysis.
		// Specimen tube with i == 0 is therefore skipped.
		if (tubeContentType == 'Specimen')
		{
			for (var i = 0; i < nofTubes; i++)
			{
				wellIndex = 3*i;
				var specWell = null;
				if (i > 0)
				{
					specWell = freeWells[wellIndex];
					if (!specWell || specWell == '') 
					{
						Wizard.setInputStatus('box.'+i, 'invalid', 'Missing box');
						return;
					}
				}
				var dnaWell = freeWells[wellIndex+1];
				if (!dnaWell || dnaWell == '') 
				{
					Wizard.setInputStatus('dnaBox.'+i, 'invalid', 'Missing box');
					return;
				}
				var rnaWell = freeWells[wellIndex+2];
				if (!rnaWell || rnaWell == '') 
				{
					Wizard.setInputStatus('rnaBox.'+i, 'invalid', 'Missing box');
					return;
				}

				// Update form fields for storage plates
				if (specWell)
				{
					frm['box.'+i].value = specWell.bioPlate.name;
					frm['row.'+i].value = Meludi.wellToAlpha(specWell.row);
					frm['column.'+i].value = specWell.column+1;
				}
			
				if (dnaWell)
				{
					frm['dnaBox.'+i].value = dnaWell.bioPlate.name;
					frm['dnaRow.'+i].value = Meludi.wellToAlpha(dnaWell.row);
					frm['dnaColumn.'+i].value = dnaWell.column+1;
				}

				if (rnaWell)
				{
					frm['rnaBox.'+i].value = rnaWell.bioPlate.name;
					frm['rnaRow.'+i].value = Meludi.wellToAlpha(rnaWell.row);
					frm['rnaColumn.'+i].value = rnaWell.column+1;
				}

				// Move focus to next field
				Events.addEventHandler('box.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'row.'+i });
				Events.addEventHandler('row.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'column.'+i });
				Events.addEventHandler('column.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'dnaBox.'+i });

				Events.addEventHandler('dnaBox.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'dnaRow.'+i });
				Events.addEventHandler('dnaRow.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'dnaColumn.'+i });
				Events.addEventHandler('dnaColumn.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'rnaBox.'+i });

				Events.addEventHandler('rnaBox.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'rnaRow.'+i });
				Events.addEventHandler('rnaRow.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'rnaColumn.'+i });
				Events.addEventHandler('rnaColumn.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'nofSections.'+i });

				Events.addEventHandler('nofSections.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'comment.'+i });
			
				// Form validation
				Events.addEventHandler('column.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('box.'+i, 'change', streg.wellOnChange, { 'boxtype': 'Specimen', 'index': i });
				Events.addEventHandler('row.'+i, 'change', streg.wellOnChange, { 'boxtype': 'Specimen', 'index': i });
				Events.addEventHandler('column.'+i, 'change', streg.wellOnChange, { 'boxtype': 'Specimen', 'index': i });
				Events.addEventHandler('box.'+i, 'keyup', streg.boxOnKeyUp, { 'boxtype': 'Specimen'});

				Events.addEventHandler('dnaColumn.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('dnaBox.'+i, 'change', streg.wellOnChange, { 'boxtype': 'DNA', 'index': i });
				Events.addEventHandler('dnaRow.'+i, 'change', streg.wellOnChange, { 'boxtype': 'DNA', 'index': i });
				Events.addEventHandler('dnaColumn.'+i, 'change', streg.wellOnChange, { 'boxtype': 'DNA', 'index': i });
				Events.addEventHandler('dnaBox.'+i, 'keyup', streg.boxOnKeyUp, { 'boxtype': 'DNA'});

				Events.addEventHandler('rnaColumn.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('rnaBox.'+i, 'change', streg.wellOnChange, { 'boxtype': 'RNA', 'index': i });
				Events.addEventHandler('rnaRow.'+i, 'change', streg.wellOnChange, { 'boxtype': 'RNA', 'index': i });
				Events.addEventHandler('rnaColumn.'+i, 'change', streg.wellOnChange, { 'boxtype': 'RNA', 'index': i });
				Events.addEventHandler('rnaBox.'+i, 'keyup', streg.boxOnKeyUp, { 'boxtype': 'RNA'});

				Events.addEventHandler('nofSections.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('nofSections.'+i, 'change', streg.nofSectionsOnChange, { 'index': i });
			
				wellIsValid[i] = true;
				dnaWellIsValid[i] = true;
				rnaWellIsValid[i] = true;
/*
				Wizard.setInputStatus('position.'+i, 'valid');
*/
				Wizard.setInputStatus('box.'+i, 'valid');
				Wizard.setInputStatus('dnaBox.'+i, 'valid');
				Wizard.setInputStatus('rnaBox.'+i, 'valid');
				if (tubeContentType == 'Specimen')
				{
					Wizard.setInputStatus('nofSections.'+i, 'valid');
				}
			}
		}
		else if (tubeContentType == 'DNA' || tubeContentType == 'RNA' || tubeContentType == 'DNA_RNA')
		{
			for (var i = 0; i < nofTubes; i++)
			{
				var wellIndex = i;
				var well = freeWells[wellIndex];
				if (!well || well == '') 
				{
					Wizard.setInputStatus('box.'+i, 'invalid', 'Missing box');
					return;
				}
				frm['box.'+i].value = well.bioPlate.name;
				frm['row.'+i].value = Meludi.wellToAlpha(well.row);
				frm['column.'+i].value = well.column+1;
			
				// Move focus to next field
				Events.addEventHandler('box.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'row.'+i });
				Events.addEventHandler('row.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'column.'+i });
				Events.addEventHandler('column.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'nofSections.'+i });
				Events.addEventHandler('nofSections.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'comment.'+i });
			
				// Form validation
				Events.addEventHandler('column.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('box.'+i, 'change', streg.wellOnChange, { 'index': i });
				Events.addEventHandler('row.'+i, 'change', streg.wellOnChange, { 'index': i });
				Events.addEventHandler('column.'+i, 'change', streg.wellOnChange, { 'index': i });
				Events.addEventHandler('box.'+i, 'keyup', streg.boxOnKeyUp);
				Events.addEventHandler('nofSections.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('nofSections.'+i, 'change', streg.nofSectionsOnChange, { 'index': i });
			
				wellIsValid[i] = true;
/*
				Wizard.setInputStatus('position.'+i, 'valid');
*/
				Wizard.setInputStatus('box.'+i, 'valid');
				if (tubeContentType == 'Specimen')
				{
					Wizard.setInputStatus('nofSections.'+i, 'valid');
				}
			}
		}
	}

	streg.fixNofTubes = function(tubeContentType, nofTubes)
	{
		if (tubeContentType == 'DNA')
		{
			nofTubes = 1;
		}
		else if (tubeContentType == 'RNA')
		{
			nofTubes = 1;
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			nofTubes = 2;
		}
		return nofTubes;
	}
	
	streg.fetchStorageBoxSuffix = function(tubeContentType)
	{
		var storageBoxSuffix = null;
		if (tubeContentType == 'Specimen')
		{
			storageBoxSuffix = '_sp';
		}
		else if (tubeContentType == 'DNA')
		{
			storageBoxSuffix = '_dna';
		}
		else if (tubeContentType == 'RNA')
		{
			storageBoxSuffix = '_rna';
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			storageBoxSuffix = '_dna_rna';
		}
		return storageBoxSuffix;
	}
	
	streg.fetchTubeContentTypeName = function(tubeContentType, tubeIndex)
	{
		var tubeContentTypeName = tubeContentType;
		if (tubeContentType == 'DNA_RNA')
		{
			if (tubeIndex == 0)
			{
				tubeContentTypeName = 'DNA';
			}
			else if (tubeIndex == 1)
			{
				tubeContentTypeName = 'RNA';
			}
		}
		return tubeContentTypeName;
	}
	
	streg.fetchTubeContentTypeNameInternal = function(tubeContentType, tubeContentTypeName)
	{
		var tubeContentTypeNameInternal = tubeContentTypeName;
		// Convert all names except abbreviations to all lower case
		if (tubeContentType == 'Specimen')
		{
			tubeContentTypeNameInternal = tubeContentTypeName.toLowerCase();
		}
		return tubeContentTypeNameInternal;
	}
	
	streg.fetchTubeName = function(tubeContentType, caseInfo, tubeIndex)
	{
		var tubeName = Strings.encodeTags(caseInfo.originalName);
		if (tubeContentType == 'Specimen')
		{
			var tubeNo = tubeIndex + caseInfo.specimenFirstIndex;
			tubeName = tubeName+'.'+tubeNo;
		}
		else if (tubeContentType == 'DNA')
		{
			tubeName = tubeName+'.d';
		}
		else if (tubeContentType == 'RNA')
		{
			tubeName = tubeName+'.r';
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			if (tubeIndex == 0)
			{
				tubeName = tubeName+'.d';
			}
			else if (tubeIndex == 1)
			{
				tubeName = tubeName+'.r';
			}
		}
		return tubeName;
	}
	
	streg.fetchTubeIdentifier = function(tubeContentType, tubeName, tubeNo)
	{
		var tubeIdentifier = tubeName;
		if (tubeContentType == 'Specimen')
		{
			tubeIdentifier = tubeNo;
		}
		return tubeIdentifier;
	}
	
	streg.boxOnKeyUp = function(event)
	{
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		
		var boxType = Data.get(event.currentTarget, 'boxtype');
		var index = Data.int(event.currentTarget, 'index');

		// Get box prefix, default is for input tubes
		var boxPrefix = 'box.';
		if (boxType && boxType == 'DNA')
		{
			boxPrefix = 'dnaBox.';
		}
		if (boxType && boxType == 'RNA')
		{
			boxPrefix = 'rnaBox.';
		}

		// Check box name
		var boxIsValid = true;
		var box = frm[boxPrefix+index].value;
		Wizard.setInputStatus(boxPrefix+index);
		// Check for prefix "ML_"
		if (box.indexOf('ML_') != 0)
		{
			boxIsValid = false;
		}
		var suffix = '_sp';
		if (tubeContentType == 'DNA' || (boxType && boxType == 'DNA'))
		{
			suffix = '_dna';
		}
		else if (tubeContentType == 'RNA' || (boxType && boxType == 'RNA'))
		{
			suffix = '_rna';
		}
		// Check for suffix
		if (box.indexOf(suffix) != (box.length - suffix.length))
		{
			boxIsValid = false;
		}
		
		if (!boxIsValid)
		{
			Wizard.setInputStatus(boxPrefix+index, 'invalid', 'Box name must start with "ML_" and end with "'+suffix+'"');
			return;
		}
	}
	
	streg.wellOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		var target = event.currentTarget;

		var boxType = Data.get(target, 'boxtype');
		var index = Data.int(target, 'index');

		// Get well position prefixes, default is for input tubes
		var boxPrefix = 'box.';
		var rowPrefix = 'row.';
		var columnPrefix = 'column.';
		if (boxType && boxType == 'DNA')
		{
			boxPrefix = 'dnaBox.';
			rowPrefix = 'dnaRow.';
			columnPrefix = 'dnaColumn.';
		}
		if (boxType && boxType == 'RNA')
		{
			boxPrefix = 'rnaBox.';
			rowPrefix = 'rnaRow.';
			columnPrefix = 'rnaColumn.';
		}
		var box = frm[boxPrefix+index].value;
		var row = frm[rowPrefix+index].value;
		var column = frm[columnPrefix+index].value;
		
		//wellIsValid[index] = true;
		if (!boxType || boxType == 'Specimen')
		{
			wellIsValid[index] = false;
		}
		else if (boxType && boxType == 'DNA')
		{
			dnaWellIsValid[index] = false;
		}
		else if (boxType && boxType == 'RNA')
		{
			rnaWellIsValid[index] = false;
		}
		else
		{
			wellIsValid[index] = false;
		}
		Wizard.setInputStatus(boxPrefix+index);
/*
		Wizard.setInputStatus('position.'+index);
*/
		
		if (box == '') 
		{
			Wizard.setInputStatus(boxPrefix+index, 'invalid', 'Missing box');
			return;
		}

		// Position errors are reported on box message line		
		if (row == '')
		{
			Wizard.setInputStatus(boxPrefix+index, 'invalid', 'Missing row');
			return;
		}

		if (column == '')
		{
			Wizard.setInputStatus(boxPrefix+index, 'invalid', 'Missing column');
			return;
		}
		
		// Check for duplicates
		var nofTubes = parseInt(frm.nofTubes.value);
		for (var i=0; i < nofTubes; i++)
		{		
			if (i != index)
			{
				if (box == frm.elements[boxPrefix+i].value && row == frm[rowPrefix+i].value && column == frm[columnPrefix+i].value)
				{
					Wizard.setInputStatus(boxPrefix+index, 'invalid', 'The position is already used once in this wizard');				
					return;
				}
			}
		}

		//Wizard.setInputStatus('position.'+index, 'checking', 'Checking...');
		Wizard.setInputStatus(boxPrefix+index, 'checking', 'Checking...');
		
		// Seems to be ok -- check if used by another sample
		var request = Ajax.getXmlHttpRequest();
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=ValidateWell';
		url += '&box=' + encodeURIComponent(box);	
		url += '&row=' + encodeURIComponent(row);
		url += '&column=' + encodeURIComponent(column);
		
		Wizard.asyncJsonRequest(url, function(response) { streg.onBoxValidated(response, boxType, index) } );
	}
	
	streg.onBoxValidated = function(response, boxType, index)
	{
		// Get well position prefixes, default is for input tubes
		var boxPrefix = 'box.';
		if (boxType && boxType == 'DNA')
		{
			boxPrefix = 'dnaBox.';
		}
		if (boxType && boxType == 'RNA')
		{
			boxPrefix = 'rnaBox.';
		}

		//Wizard.setInputStatus('position.'+index);
		Wizard.setInputStatus(boxPrefix+index);
		if (response.message)
		{
			//Wizard.setInputStatus(response.box ? 'box.'+index : 'position.'+index, 'invalid', Strings.encodeTags(response.message));
			Wizard.setInputStatus(boxPrefix+index, 'invalid', Strings.encodeTags(response.message));
			return;
		}
		
		//wellIsValid[index] = true;
		if (!boxType || boxType == 'Specimen')
		{
			wellIsValid[index] = true;
		}
		else if (boxType && boxType == 'DNA')
		{
			dnaWellIsValid[index] = true;
		}
		else if (boxType && boxType == 'RNA')
		{
			rnaWellIsValid[index] = true;
		}
		else
		{
			wellIsValid[index] = true;
		}
		Wizard.setInputStatus(boxPrefix+index, 'valid');
/*
		Wizard.setInputStatus('position.'+index, 'valid');
*/
	}

	
	streg.nofSectionsOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		var target = event.currentTarget;

		var index = Data.int(target, 'index');
		var nofSections = frm['nofSections.'+index].value;

		Wizard.setInputStatus('nofSections.'+index);

		if (nofSections == '')
		{
			Wizard.setInputStatus('nofSections'+index, 'invalid', 'Missing');
			return;
		}
		if (parseInt(nofSections) < 1)
		{
			Wizard.setInputStatus('nofSections'+index, 'invalid', 'Must be at least 1');
			return;		
		}
		Wizard.setInputStatus('nofSections'+index, 'valid');
	}

	streg.validateStep3 = function(event)
	{
		var valid = false;

		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		if (tubeContentType == 'Specimen')
		{
			for (var i = 0; i < wellIsValid.length; i++)
			{
				// No storage space is allocated for first specimen,
				// since all amount is expected to be used for analysis.
				// Specimen tube with i == 0 is therefore skipped.
				if (i > 0)
				{
					if (!wellIsValid[i])
					{
						event.preventDefault();
						return;
					}
				}
				if (!dnaWellIsValid[i])
				{
					event.preventDefault();
					return;
				}
				if (!rnaWellIsValid[i])
				{
					event.preventDefault();
					return;
				}
			}
		}
		else
		{
			// Input DNA, RNA, or DNA + RNA
			for (var i = 0; i < wellIsValid.length; i++)
			{
				if (!wellIsValid[i])
				{
					event.preventDefault();
					return;
				}
			}
		}
	}

	
	streg.submit = function()
	{
		var frm = document.forms['meludi'];

		caseInfo.arrivalDate = Strings.trim(frm.arrivalDate.value);
		caseInfo.samplingDate = Strings.trim(frm.samplingDate.value);
		caseInfo.mutationAnalysisKras = frm.mutationAnalysisKras.checked;
		caseInfo.mutationAnalysisNras = frm.mutationAnalysisNras.checked;
		caseInfo.mutationAnalysisBraf = frm.mutationAnalysisBraf.checked;
		caseInfo.mutationAnalysisEgfr = frm.mutationAnalysisEgfr.checked;
		caseInfo.mutationAnalysisAlkEml4 = frm.mutationAnalysisAlkEml4.checked;
		caseInfo.mutationAnalysisPdgfra = frm.mutationAnalysisPdgfra.checked;
		caseInfo.mutationAnalysisKit = frm.mutationAnalysisKit.checked;
		caseInfo.mutationAnalysisOther = frm.mutationAnalysisOther.checked;
		caseInfo.site = frm.site.value;
		caseInfo.tubeContentType = frm.tubeContentType.value;
		caseInfo.specimenType = frm.specimenType.value;
		caseInfo.projectFocus = frm.projectFocus.value;
		caseInfo.viableTumourCellsPercent = frm.viableTumourCellsPercent.value;
		caseInfo.otherPathNote = frm.otherPathNote.value;
		
		var nofTubes = parseInt(frm.nofTubes.value);
		caseInfo.specimen = [];
		for (var i = 0; i < nofTubes; i++)
		{
			var specimenTube ={};
			specimenTube.name = streg.fetchTubeName(caseInfo.tubeContentType, caseInfo, i);
			specimenTube.box = frm['box.'+i].value;
			specimenTube.row = frm['row.'+i].value;
			specimenTube.column = frm['column.'+i].value;
			var nofSections = null;
			if (caseInfo.tubeContentType == 'Specimen')
			{
				// Storage position for DNA extract from specimen
				specimenTube.dnaBox = frm['dnaBox.'+i].value;
				specimenTube.dnaRow = frm['dnaRow.'+i].value;
				specimenTube.dnaColumn = frm['dnaColumn.'+i].value;

				// Storage position for RNA extract from specimen				
				specimenTube.rnaBox = frm['rnaBox.'+i].value;
				specimenTube.rnaRow = frm['rnaRow.'+i].value;
				specimenTube.rnaColumn = frm['rnaColumn.'+i].value;
				
				nofSections = frm['nofSections.'+i].value;
			}
			specimenTube.nofSections = nofSections;
			specimenTube.comment = frm['comment.'+i].value;
			specimenTube.specimenType = frm.specimenType.value;
			caseInfo.specimen[i] = specimenTube;
		}

		var submitInfo = {};
		submitInfo.caseInfo = caseInfo;

		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateSpecimenTubes';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, streg.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	streg.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
		Doc.show('gonext-message');
	}

	streg.countDownToRestart = function()
	{
		var time = restartAt - (new Date()).getTime();
		if (time < 0)
		{
			Wizard.restartWizard();
		}
		else
		{
			Doc.element('gonext-message').innerHTML = '<i>Wizard will restart automatically in ' + Math.ceil(time/1000) + ' seconds...</i>';
		}
	}
	
	streg.stopAutoRestart = function()
	{
		clearTimeout(restartTimer);
		Doc.hide('gonext-message');
		Doc.hide('gostop');
	}

	streg.getCheckboxStatus = function(checkbox)
	{
		return checkbox.checked;
	}
	
	return streg;
}();

Doc.onLoad(SpecimenTubeReg.initPage);

